""" serial module for EPOC

The serial module wraps EPOC's _serial module, to provide
thread-independent serial ports.  If you can get away with
explicitely opening ports in all your threads, it'll be more
efficient

"""

import _serial
import threading

PortClosed = "PortClosed"

class _Port:
    def __init__(self, portname):
        self.__portname = portname
        self.__open = 1
    def read(self, length):
        if not self.__open:
            self._check()
            raise PortClosed
        while 1:
            try:
                return threading.currentThread().__ports[self.__portname].read(length)
            except AttributeError:
                # - __ports does not exist
                threading.currentThread().__ports = {}
            except KeyError:
                # - No entry for our portname
                threading.currentThread().__ports[self.__portname] = _serial.open(self.__portname)
    def write(self, data):
        if not self.__open:
            self._check()
            raise PortClosed
        while 1:
            try:
                threading.currentThread().__ports[self.__portname].write(data)
                return
            except AttributeError:
                # - __ports does not exist
                threading.currentThread().__ports = {}
            except KeyError:
                # - No entry for our portname
                threading.currentThread().__ports[self.__portname] = _serial.open(self.__portname)
    def close(self):
        self.__open = 0
	self._check()

    def _check(self):
        if not self.__open:
            try:
            	t = threading.currentThread()
            	if t.__ports.has_key(self.__portname_):
            	    del t.__ports[self.__portname]
            	    if not t.__ports:
            	        del t.__ports
            except AttributeError:
                pass
    def __del__(self):
        self.close()

def open(portname):
    """ Open the named port/unit """
    return _Port(portname)
